﻿Imports System.Runtime.Serialization.Formatters.Binary
Imports ThinkGeo.MapSuite
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.WinForms

<Serializable()>
Public Class CustomMapPrinterLayer
	Inherits MapPrinterLayer
	Private mbIsSelected As Boolean = False
	Public Property PageLayoutMapNumber As Integer = 0
	Public Property SyncScaleWithParentMap As Boolean = True
	Public Property UnSelectedBackGroundMask As AreaStyle
		Get
			Return moUnSelectedBackgroundMask
		End Get
		Set(value As AreaStyle)
			moUnSelectedBackgroundMask = value
		End Set
	End Property
	Private moSelectedBackgroundMask As AreaStyle
	Private moUnSelectedBackgroundMask As AreaStyle
	Public Property SelectedBackgroundMask As AreaStyle
		Get
			If moSelectedBackgroundMask Is Nothing Then
				moSelectedBackgroundMask = New AreaStyle
			End If
			Return moSelectedBackgroundMask
		End Get
		Set(value As AreaStyle)
			moSelectedBackgroundMask = value
		End Set
	End Property
	<NonSerialized()>
	Private moLegend As LegendPrinterLayer
	<NonSerialized()>
	Private moParentMap As WinformsMap
	Public Enum EnumMaskType
		NormalMask = 1
		SelectedMask = 2
	End Enum
	Public Enum EnumSelectionType
		Selected = 1
		UnSelected = 2
	End Enum
	Public Property IsSelected As Boolean
		Get
			Return mbIsSelected
		End Get
		Set(value As Boolean)
			mbIsSelected = value
		End Set
	End Property
	Public Sub New()
		MyBase.New()
		mInitSymbology()
	End Sub
	Public Sub New(oLayers As IEnumerable(Of Layer), oExtent As RectangleShape, eUnit As GeographyUnit)
		MyBase.New(oLayers, oExtent, eUnit)
		mInitSymbology()
	End Sub
	Public Sub UnSelectMap()
		If Not UnSelectedBackGroundMask Is Nothing Then
			BackgroundMask = CType(UnSelectedBackGroundMask.CloneDeep, AreaStyle)
			mbIsSelected = False
		End If
	End Sub
	Public Function Clone() As CustomMapPrinterLayer
		Dim oObj As Object
		Using oStream As New IO.MemoryStream()
			Dim oFormatter As New BinaryFormatter()
			oFormatter.Serialize(oStream, Me)
			oStream.Position = 0
			oObj = oFormatter.Deserialize(oStream)
			oStream.Close()
		End Using
		Return CType(oObj, CustomMapPrinterLayer)
	End Function
	Public Sub SelectMap()
		'***** Set the background mask to indicate that this Page Layout map
		'***** is the target map
		If Not mbIsSelected Then
			UnSelectedBackGroundMask = CType(BackgroundMask.CloneDeep, AreaStyle)
			BackgroundMask = CType(SelectedBackgroundMask.CloneDeep, AreaStyle)
			If Not ParentMap Is Nothing Then
				'If Not ParentMap.IsActiveMap Then
				'	ParentMap.Toc.ActivateMap(ParentMap)
				'End If
			End If
			mbIsSelected = True
		End If
	End Sub
	Public ReadOnly Property CurrentScale As Double
		Get
			Open()
			Dim dWidth As Single = CSng(GetPosition().Width)
			Dim dHeight As Single = CSng(GetPosition().Height)
			Return ExtentHelper.GetScale(MapExtent, dWidth, MapUnit)
		End Get
	End Property
	Public Property ParentMap As WinformsMap
		Get
			Return moParentMap
		End Get
		Set(value As WinformsMap)
			moParentMap = value
		End Set
	End Property
	Public Property Legend As LegendPrinterLayer
		Get
			Return moLegend
		End Get
		Set(value As LegendPrinterLayer)
			moLegend = value
		End Set
	End Property
	Protected Overrides Sub DrawCore(canvas As ThinkGeo.MapSuite.Drawing.GeoCanvas, labelsInAllLayers As System.Collections.ObjectModel.Collection(Of SimpleCandidate))
		'***** Make sure the DrawingLevel is set to LevelOne below, if not the background 
		'***** mask may print on top of the map layers, making them invisible.
		'***** SET 11/12/2015
		'***** QB #2003
		If IsSelected Then
			'***** Map appears with double-dash outline 
			BackgroundMask = New CustomAreaStyle(CType(UnSelectedBackGroundMask.CloneDeep(), AreaStyle).FillSolidBrush)
			BackgroundMask.DrawingLevel = Drawing.DrawingLevel.LevelOne
		Else
			'***** For printing or when a map is not selected. Note that if a page layout
			'***** only has one map, that map is always selected.
			BackgroundMask = CType(UnSelectedBackGroundMask.CloneDeep(), AreaStyle)
			BackgroundMask.DrawingLevel = Drawing.DrawingLevel.LevelOne
		End If
		MyBase.DrawCore(canvas, labelsInAllLayers)
	End Sub
	Private Sub mInitSymbology()
		moSelectedBackgroundMask = New CustomAreaStyle(New GeoSolidBrush(GeoColor.SimpleColors.White))
		UnSelectedBackGroundMask = Form1.CreateSimpleAreaStyle(GeoColor.SimpleColors.White, GeoColor.SimpleColors.Black, 1)
	End Sub
End Class
