﻿Imports ThinkGeo.MapSuite
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.WinForms
Imports ThinkGeo.MapSuite.Serialize

Public Class Form1
	Private moPolyLayer As ShapeFileFeatureLayer
	Private moImagePrinterLayer As ImagePrinterLayer
	Public Enum EnumPagePosition
		TopLeft = 1
		TopRight = 2
		LowerLeft = 3
		LowerRight = 4
	End Enum
	Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
		CreateDefaultLayout()
	End Sub
	Private Sub btnSerialize_Click(sender As Object, e As EventArgs) Handles btnSerialize.Click
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oImagePrinterLayer As ImagePrinterLayer
		Dim sSavePath As String = "..\..\Data\ImageLayer.xml"
		If oPrinterOverlay.PrinterLayers.Contains("ImagePrinterLayer") Then
			oImagePrinterLayer = oPrinterOverlay.PrinterLayers("ImagePrinterLayer")
			oImagePrinterLayer.Close()
			oPrinterOverlay.PrinterLayers.Remove("ImagePrinterLayer")
			Dim oSerializer As New GeoSerializer
			oSerializer.Serialize(oImagePrinterLayer, sSavePath)
			Me.PageLayout.Refresh()
		End If
	End Sub

	Private Sub btnDeserialize_Click(sender As Object, e As EventArgs) Handles btnDeserialize.Click
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oImagePrinterLayer As ImagePrinterLayer
		Dim sSavePath As String = "..\..\Data\ImageLayer.xml"
		If Not oPrinterOverlay.PrinterLayers.Contains("ImagePrinterLayer") Then
			Dim oDeSerializer As New GeoSerializer
			oImagePrinterLayer = CType(oDeSerializer.Deserialize(sSavePath, GeoFileReadWriteMode.Read), ImagePrinterLayer)
			oPrinterOverlay.PrinterLayers.Add("ImagePrinterLayer", oImagePrinterLayer)
			Try
				Me.PageLayout.Refresh()
			Catch ex As Exception
				MessageBox.Show("Error: " + ex.Message, "Boom!", MessageBoxButtons.OK, MessageBoxIcon.Error)
			End Try

		End If
	End Sub
	Public Sub CreateDefaultLayout()
		Dim oExtentRectangle As RectangleShape
		Dim moPrinterOverlay As PrinterInteractiveOverlay
		Dim moPrinterPage As PagePrinterLayer

		moPrinterOverlay = New PrinterInteractiveOverlay()
		With PageLayout
			'***** Setup the map unit, you want to always use feet or meters for printer layout maps
			.MapUnit = GeographyUnit.Feet
			'***** Create the default zoom levels for the layout map
			.ZoomLevelSet = New PrinterZoomLevelSet(.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(.MapUnit))

			'***** Set the background color to gray so there is contrast with the white page
			.BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

			'***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
			'***** The interactive overlay allows the layers to be interacted with
			.InteractiveOverlays.Add("PrinterOverlay", moPrinterOverlay)
			.InteractiveOverlays.MoveToBottom("PrinterOverlay")

			'***** Create the PagePrinterLayer which shows the page boundary and 
			'***** is the area the user will arrange all of the other layers on top of.
			moPrinterPage = New PagePrinterLayer()
			With moPrinterPage
				.Open()
				.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)
				moPrinterOverlay.PrinterLayers.Add("PrinterPage", moPrinterPage)
				'***** Get the page's extent, scale it up by 10%
				oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
				'***** Set oExtentRectangle as the default current extent
			End With
			.CurrentExtent = oExtentRectangle

			'***** Set the minimum scale of the map to the last zoom level
			.MinimumScale = .ZoomLevelSet.ZoomLevel20.Scale

			'***** Add a container for the map layers
			AddMapLayerContainer(mGetPageLayoutMap)

			AddImageLayer(mGetPageLayoutMap)
		End With
	End Sub
	Private Sub AddImageLayer(oMapPrinterLayer As MapPrinterLayer)
		Dim sImagePath As String = "..\..\Data\SmallSilvicsLogo.jpg"
		Dim oImage As New GeoImage(sImagePath)
		moImagePrinterLayer = New ImagePrinterLayer(oImage)
		Dim dAspRat As Double = oImage.Width / oImage.Height
		Dim dWidth As Double = 2
		Dim dHeight As Double = 2 / dAspRat
		SetPosition(dWidth, dHeight, moImagePrinterLayer, EnumPagePosition.TopLeft, mGetPageLayoutMap)
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		oPrinterOverlay.PrinterLayers.Add("ImagePrinterLayer", moImagePrinterLayer)
	End Sub
	Private Sub Form1_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown
		Me.PageLayout.Refresh()
	End Sub
	Public Sub AddMapLayerContainer(moPageLayoutMap As MapPrinterLayer)
		Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)

		moPageLayoutMap = New MapPrinterLayer()
		moPageLayoutMap.MapUnit = GeographyUnit.Feet

		'***** Create the MapPrinterLayer and set the position
		moPageLayoutMap.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)

		'***** Set the maps position slightly below the pages center and 7.5 inches wide and 9 inches tall
		oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
		moPageLayoutMap.SetPosition(7.5, 9, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y - 0.25, PrintingUnit.Inch)

		'***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
		Dim moPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		moPrinterOverlay.PrinterLayers.Add("MapPrinterLayer", moPageLayoutMap)
	End Sub
	Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
		Dim moPrinterPage As PagePrinterLayer = mGetPrinterPage()
		Return moPrinterPage.GetPosition(unit)
	End Function
	Public Sub SetPosition(ByVal dWidth As Double, ByVal dHeight As Double, ByVal oPrinterLayer As PrinterLayer, ByVal eInitialPosition As EnumPagePosition, moPageLayoutMap As MapPrinterLayer)
		Dim dCenterPointX As Double
		Dim dCenterPointY As Double
		Dim dDistanceFromEdge As Double = 0.25

		'***** Get the bounding box for the MapPrinterLayer
		moPageLayoutMap.Open()
		Dim oMapBoundingBox As RectangleShape = moPageLayoutMap.GetPosition(PrintingUnit.Inch)
		Select Case eInitialPosition
			Case EnumPagePosition.LowerRight
				dCenterPointX = oMapBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
			Case EnumPagePosition.LowerLeft
				dCenterPointX = oMapBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
			Case EnumPagePosition.TopRight
				dCenterPointX = oMapBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
			Case EnumPagePosition.TopLeft
				dCenterPointX = oMapBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
		End Select

		'***** Now let's try to place this layer 
		oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)
	End Sub
	Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
		mRefreshMapLayers(mGetPageLayoutMap)
	End Sub
	Private Sub mRefreshMapLayers(moPageLayoutMap As MapPrinterLayer)
		If Not moPageLayoutMap Is Nothing Then
			moPageLayoutMap.IsDrawing = True
		End If

		PageLayout.Refresh()

		If Not moPageLayoutMap Is Nothing Then
			moPageLayoutMap.IsDrawing = False
		End If
	End Sub
	Private Function mGetPrinterOverlay() As PrinterInteractiveOverlay
		For Each oPrinterOverlay In PageLayout.InteractiveOverlays
			If TypeOf oPrinterOverlay Is PrinterInteractiveOverlay Then
				Return CType(oPrinterOverlay, PrinterInteractiveOverlay)
			End If
		Next oPrinterOverlay
		Return Nothing
	End Function
	Private Function mGetPageLayoutMap() As MapPrinterLayer
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oPrinterLayer As PrinterLayer
		For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
			If TypeOf oPrinterLayer Is MapPrinterLayer Then
				Return CType(oPrinterLayer, MapPrinterLayer)
			End If
		Next oPrinterLayer
		Return Nothing
	End Function
	Private Function mGetPrinterPage() As PagePrinterLayer
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oPrinterLayer As PrinterLayer
		For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
			If TypeOf oPrinterLayer Is PagePrinterLayer Then
				Return CType(oPrinterLayer, PagePrinterLayer)
			End If
		Next oPrinterLayer
		Return Nothing
	End Function

End Class
